import { DynamicModule, Global, Inject, MiddlewareConsumer, Module, NestModule } from '@nestjs/common';
import morgan from 'morgan';
import Transport from 'winston-transport';

import { LoggerNestAdapter } from './logger.nest.adapter';
import { LoggerService } from './logger.service';
import { Logger, LoggerBaseKey, LoggerKey, LoggerModuleOptions } from './logger.types';
import ConsoleTransport from './winston/transports/console.transport';
import FileTransport from './winston/transports/file.transport';
import SlackTransport from './winston/transports/slack.transport';
import WinstonLogger, { WinstonLoggerTransportsKey } from './winston/winston.logger';

@Global()
@Module({})
export class LoggerModule implements NestModule {
	public constructor(@Inject(LoggerKey) private logger: Logger) {}

	public configure(consumer: MiddlewareConsumer): void {
		consumer
			.apply(
				morgan('combined', {
					stream: {
						write: (message: string) => {
							this.logger.debug(message, 'RequestLogger');
						},
					},
				})
			)
			.forRoutes('*');
	}

	public static forRoot(options: LoggerModuleOptions): DynamicModule {
		return {
			module: LoggerModule,
			imports: [],
			controllers: [],
			providers: [
				{
					provide: LoggerBaseKey,
					useClass: WinstonLogger,
				},
				{
					provide: LoggerKey,
					useClass: LoggerService,
				},
				{
					provide: LoggerNestAdapter,
					useFactory: (logger: Logger) => new LoggerNestAdapter(logger),
					inject: [LoggerKey],
				},

				{
					provide: WinstonLoggerTransportsKey,
					useFactory: () => {
						const transports: Transport[] = [];

						transports.push(ConsoleTransport.create(options.appName));

						transports.push(FileTransport.create());

						transports.push(SlackTransport.create(options.slack.webhook.url, options.nodeEnv));

						return transports;
					},
				},
			],
			exports: [LoggerKey, LoggerNestAdapter],
		};
	}
}
